package org.apithefire.servlet.dynamic;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.apithefire.servlet.util.ServletUtil;

public abstract class AbstractDynamicFilter implements DynamicFilter {
	
	private Filter filterImpl;

	public final synchronized void init(FilterConfig config)
			throws ServletException {
		init(config.getServletContext());
	}

	public synchronized void init(ServletContext servletContext)
			throws ServletException {
		if (filterImpl == null) {
			filterImpl = newFilterImpl();
			filterImpl.init(new DynamicFilterConfig(this, servletContext));
		}
	}
	
	protected abstract Filter newFilterImpl();

	public Map<String, String> getInitParamMap() {
		return Collections.emptyMap();
	}

	public synchronized void destroy() {
		if (filterImpl != null) {
			filterImpl.destroy();
			filterImpl = null;
		}
	}

	public synchronized void doFilter(ServletRequest request,
			ServletResponse response, FilterChain chain) throws IOException,
			ServletException {
		if (filterImpl == null) {
			HttpServletRequest httpRequest = ServletUtil.httpRequest(request);
			
			init(httpRequest.getSession().getServletContext());
		}
		filterImpl.doFilter(request, response, chain);
	}

}
