package org.apithefire.servlet.dynamic;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.apithefire.servlet.util.ServletUtil;

public abstract class AbstractDynamicServlet implements DynamicServlet {
	
	private Servlet servletImpl;
	private ServletConfig config;

	public String getServletInfo() {
		return toString();
	}

	public final synchronized void init(ServletConfig config)
			throws ServletException {
		init(config.getServletContext());
	}

	public synchronized void init(ServletContext servletContext)
			throws ServletException {
		if (servletImpl == null) {
			servletImpl = newServletImpl();
			config = new DynamicServletConfig(this, servletContext);
			servletImpl.init(config);
		}
	}
	
	protected abstract Servlet newServletImpl();

	public Map<String, String> getInitParamMap() {
		return Collections.emptyMap();
	}

	public synchronized void destroy() {
		if (servletImpl != null) {
			servletImpl.destroy();
			servletImpl = null;
			config = null;
		}
	}

	public synchronized ServletConfig getServletConfig() {
		return config;
	}

	public synchronized void service(ServletRequest request,
			ServletResponse response) throws ServletException, IOException {
		if (servletImpl == null) {
			HttpServletRequest httpRequest = ServletUtil.httpRequest(request);
			
			init(httpRequest.getSession().getServletContext());
		}
		servletImpl.service(request, response);
	}

}
