package org.apithefire.servlet.dynamic;

import java.util.Collections;
import java.util.Enumeration;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;

import org.apithefire.util.lang.Objects;

public class DynamicServletConfig implements ServletConfig {

	private final DynamicServlet dynamicServlet;
	private final ServletContext servletContext;
	private final Map<String, String> initParamMap;

	public DynamicServletConfig(DynamicServlet dynamicServlet,
			ServletContext servletContext) {
		this.dynamicServlet = Objects.nonNull(dynamicServlet);
		this.servletContext = Objects.nonNull(servletContext);
		this.initParamMap = Objects.nonNull(dynamicServlet.getInitParamMap());
	}

	public String getServletName() {
		return dynamicServlet.toString();
	}

	public ServletContext getServletContext() {
		return servletContext;
	}

	public Enumeration<String> getInitParameterNames() {
		return Collections.enumeration(initParamMap.keySet());
	}

	public String getInitParameter(String name) {
		return initParamMap.get(Objects.nonNull(name));
	}

}